==> localstack-ext-1.4.1.dev20230217162849.tar.gz: localstack-ext-1.4.1.dev20230217162849/localstack_ext/bootstrap/pods/utils/metamodel_utils.py <==
_A=False
import json,logging,os,shutil,zipfile
from typing import Dict,List,Optional,Tuple
import requests
from localstack.config import get_edge_url
from localstack.utils.files import load_file,mkdir,new_tmp_dir,rm_rf,save_file
from localstack.utils.strings import to_str
from localstack_ext.bootstrap.pods.constants import COMPRESSION_FORMAT
from localstack_ext.bootstrap.pods.models import Revision,Version
from localstack_ext.bootstrap.pods.object_storage import ObjectStorageProvider
from localstack_ext.bootstrap.pods.utils.common import PodsConfigContext,add_file_to_archive,read_file_from_archive
from localstack_ext.constants import API_PATH_PODS
LOG=logging.getLogger(__name__)
PLACEHOLDER_NO_CHANGE={'_meta_':'no-change'}
class CommitMetamodelUtils:
	def __init__(A,config_context,object_storage=None):
		B=object_storage;from localstack_ext.bootstrap.pods.object_storage import get_object_storage_provider as C;A.config_context=config_context;A.object_storage=B
		if B is None:A.object_storage=C(A.config_context)
	def create_metamodel_archive(D,version,overwrite=_A,metamodels_file=None):
		E=overwrite;A=version;B=A.revisions[0]if E else A.revisions[-1];C=D.config_context.metadata_dir(A.version_number);mkdir(C)
		if metamodels_file:
			F=D.config_context.get_version_meta_archive_path(A.version_number)
			if os.path.isfile(F):
				with zipfile.ZipFile(F)as H:H.extractall(C)
		while B:
			G=B.assoc_commit
			if not G:break
			I=D.create_metamodel_delta(A,revision=B);J=D.config_context.commit_metamodel_file(B.revision_number);K=os.path.join(D.config_context.pod_root_dir,C,J);save_file(K,json.dumps(I));L=G.head_ptr if E else B.parent_ptr;B=A.get_revision(L)
		shutil.make_archive(C,COMPRESSION_FORMAT,root_dir=C);rm_rf(C)
	def create_metamodel_from_state_files(B,version):
		C=new_tmp_dir();A=B.config_context.get_version_state_archive(version=version)
		if not A:return
		with zipfile.ZipFile(A)as D:D.extractall(C)
	@classmethod
	def get_metamodel_from_instance(B):
		A=requests.get(f"{get_edge_url()}{API_PATH_PODS}/state/metamodel",verify=_A)
		if not A.ok:raise Exception(f"Unable to fetch metamodel from instance (status {A.status_code})")
		A=json.loads(to_str(A.content));return A
	@classmethod
	def get_metamodel_delta(N,prev_metamodel,this_metamodel):
		D=prev_metamodel;A=this_metamodel
		if not D:return A
		def I(prev_service_state,service_state):return service_state!=prev_service_state
		B={};A=A or{}
		for (E,J) in A.items():
			B[E]={}
			for (F,K) in J.items():
				B[E][F]=G={};L=D.get(F)or{}
				for (C,H) in K.items():
					G[C]=PLACEHOLDER_NO_CHANGE;M=L.get(C)
					if I(M,H):G[C]=H
		return B
	def create_metamodel_delta(A,version,revision,store_to_zip=_A):
		E=store_to_zip;C=revision;B=version;D=A.reconstruct_metamodel(version=B,revision=C);F=A.config_context.get_version_meta_archive_path(version=B.version_number);G=A.config_context.metamodel_file(revision=C.revision_number)
		if C.revision_number<=Revision.DEFAULT_INITIAL_REVISION_NUMBER:
			if E:add_file_to_archive(F,entry_name=G,content=json.dumps(D))
			return D
		I=B.get_revision(C.revision_number-1);J=A.reconstruct_metamodel(version=B,revision=I);H=A.get_metamodel_delta(J,D)
		if E:add_file_to_archive(F,entry_name=G,content=json.dumps(H))
		return H
	def reconstruct_metamodel(A,version,revision):
		B=version;C=[]
		for D in range(Revision.DEFAULT_INITIAL_REVISION_NUMBER,revision.revision_number+1):E=A.get_version_metamodel(version=B,revision=B.get_revision(D));C.append(E or{})
		return A.reconstruct_metamodel_from_list(C)
	@classmethod
	def reconstruct_metamodel_from_list(I,metamodels):
		A={}
		for C in metamodels:
			C=C or{}
			for (B,E) in C.items():
				if B not in A:A[B]=E;continue
				for (D,F) in E.items():
					if D not in A:A[B][D]=F;continue
					for (H,G) in F.items():
						if G==PLACEHOLDER_NO_CHANGE:continue
						A[B][D][H]=G
		return A
	def get_version_metamodel(A,version,revision):B=A.object_storage.get_state_file_location_by_key(revision.metamodel_file);C=load_file(B);D=json.loads(C or'{}');return D
	def get_commit_diff(B,version_no,commit_no):
		C=version_no;D=B.config_context.get_version_meta_archive(C)
		if not D:LOG.warning('No metadata found for version %s',C);return
		E=B.config_context.commit_metamodel_file(commit_no);A=read_file_from_archive(archive_path=D,file_name=E);A=json.loads(to_str(A or'{}'));return A

==> localstack-ext-2.0.0.tar.gz: localstack-ext-2.0.0/localstack_ext/bootstrap/pods/utils/metamodel_utils.py <==
import json,logging,os,shutil,zipfile
from typing import Dict,List,Optional,Tuple
from localstack.utils.files import load_file,mkdir,new_tmp_dir,rm_rf,save_file
from localstack.utils.strings import to_str
from localstack_ext.bootstrap.pods.constants import COMPRESSION_FORMAT
from localstack_ext.bootstrap.pods.models import Revision,Version
from localstack_ext.bootstrap.pods.object_storage import ObjectStorageProvider
from localstack_ext.bootstrap.pods.utils.common import PodsConfigContext,add_file_to_archive,read_file_from_archive
LOG=logging.getLogger(__name__)
PLACEHOLDER_NO_CHANGE={'_meta_':'no-change'}
class CommitMetamodelUtils:
	def __init__(A,config_context,object_storage=None):
		B=object_storage;from localstack_ext.bootstrap.pods.object_storage import get_object_storage_provider as C;A.config_context=config_context;A.object_storage=B
		if B is None:A.object_storage=C(A.config_context)
	def create_metamodel_archive(C,version,overwrite=False,metamodels_file=None):
		E=overwrite;D=version;A=D.revisions[0]if E else D.revisions[-1];B=C.config_context.metadata_dir(D.version_number);mkdir(B)
		if metamodels_file:
			F=C.config_context.get_version_meta_archive_path()
			if os.path.isfile(F):
				with zipfile.ZipFile(F)as H:H.extractall(B)
		while A:
			G=A.assoc_commit
			if not G:break
			I=C.create_metamodel_delta(D,revision=A);J=C.config_context.commit_metamodel_file(A.revision_number);K=os.path.join(C.config_context.pod_root_dir,B,J);save_file(K,json.dumps(I));L=G.head_ptr if E else A.parent_ptr;A=D.get_revision(L)
		shutil.make_archive(B,COMPRESSION_FORMAT,root_dir=B);rm_rf(B)
	def create_metamodel_from_state_files(B,version):
		C=new_tmp_dir();A=B.config_context.get_version_state_archive()
		if not A:return
		with zipfile.ZipFile(A)as D:D.extractall(C)
	@classmethod
	def get_metamodel_delta(N,prev_metamodel,this_metamodel):
		D=prev_metamodel;A=this_metamodel
		if not D:return A
		def I(prev_service_state,service_state):return service_state!=prev_service_state
		B={};A=A or{}
		for (E,J) in A.items():
			B[E]={}
			for (F,K) in J.items():
				B[E][F]=G={};L=D.get(F)or{}
				for (C,H) in K.items():
					G[C]=PLACEHOLDER_NO_CHANGE;M=L.get(C)
					if I(M,H):G[C]=H
		return B
	def create_metamodel_delta(A,version,revision,store_to_zip=False):
		E=store_to_zip;C=version;B=revision;D=A.reconstruct_metamodel(version=C,revision=B);F=A.config_context.get_version_meta_archive_path();G=A.config_context.metamodel_file(revision=B.revision_number)
		if B.revision_number<=Revision.DEFAULT_INITIAL_REVISION_NUMBER:
			if E:add_file_to_archive(F,entry_name=G,content=json.dumps(D))
			return D
		I=C.get_revision(B.revision_number-1);J=A.reconstruct_metamodel(version=C,revision=I);H=A.get_metamodel_delta(J,D)
		if E:add_file_to_archive(F,entry_name=G,content=json.dumps(H))
		return H
	def reconstruct_metamodel(A,version,revision):
		B=version;C=[]
		for D in range(Revision.DEFAULT_INITIAL_REVISION_NUMBER,revision.revision_number+1):E=A.get_version_metamodel(version=B,revision=B.get_revision(D));C.append(E or{})
		return A.reconstruct_metamodel_from_list(C)
	@classmethod
	def reconstruct_metamodel_from_list(I,metamodels):
		A={}
		for C in metamodels:
			C=C or{}
			for (B,E) in C.items():
				if B not in A:A[B]=E;continue
				for (D,F) in E.items():
					if D not in A:A[B][D]=F;continue
					for (H,G) in F.items():
						if G==PLACEHOLDER_NO_CHANGE:continue
						A[B][D][H]=G
		return A
	def get_version_metamodel(A,version,revision):B=A.object_storage.get_state_file_location_by_key(revision.metamodel_file);C=load_file(B);D=json.loads(C or'{}');return D
	def get_commit_diff(B,version_no,commit_no):
		C=B.config_context.get_version_meta_archive()
		if not C:LOG.warning('No metadata found for version %s',version_no);return
		D=B.config_context.commit_metamodel_file(commit_no);A=read_file_from_archive(archive_path=C,file_name=D);A=json.loads(to_str(A or'{}'));return A
